@@ -90,7 +90,7 @@ static MagnifierOptions global_options = { ":0.0",
 					   10,
 					   10,
 					   0,
-					   0,
+					   1,
 					   0,
 					   0,
 					   0
@@ -151,18 +151,18 @@ magnifier_main_test_image (gpointer data)
 	Bonobo_PropertyBag properties;
 	CORBA_Environment ev;
 	GNOME_Magnifier_RectBounds roi;
-	float  since_last_timing, timing_val;
 	int x_roi, y_roi;
 
 	/* Only iterate the number of times specified */
 	if (global_options.timing_iterations > 0) {
 		if (timing_counter > global_options.timing_iterations) {
+  			CORBA_exception_init (&ev);
 	  		properties = GNOME_Magnifier_ZoomRegion_getProperties (zoom_region, &ev);
 			if (BONOBO_EX (&ev))
 				fprintf (stderr, "EXCEPTION\n");
 
 	  		bonobo_pbclient_set_boolean (properties, "exit-magnifier",
-				       global_options.timing_output, &ev);
+				       TRUE, &ev);
 		}
 	}
 
@@ -372,6 +372,11 @@ main (int argc, char** argv)
 	  putenv (string);
   }
 
+  if (global_options.timing_pan_rate && global_options.timing_iterations == 0)
+  {
+    g_error ("Must specify timing_iterations when running pan test");
+  }
+
   /* FIXME */
   gtk_init (&argc, &argv);
 
@@ -497,47 +502,47 @@ main (int argc, char** argv)
   }
 
   if (global_options.timing_pan_rate)
-    {
-      GNOME_Magnifier_ZoomRegionList *zoom_regions;
-      GNOME_Magnifier_RectBounds roi;
-      roi.x1 = 100;
-      roi.x2 = 100 + (screen_width / global_options.zoom_factor);
-      roi.y1 = 0;
-      roi.y2 = screen_height / global_options.zoom_factor;
-
-      zoom_regions = GNOME_Magnifier_Magnifier_getZoomRegions (
-        BONOBO_OBJREF (magnifier), &ev);
-
-      if (zoom_regions && (zoom_regions->_length > 0))
-        {
-          GNOME_Magnifier_ZoomRegion_setROI (
-            zoom_regions->_buffer[0], &roi, &ev);
-        }
-    }
+  {
+	  GNOME_Magnifier_ZoomRegionList *zoom_regions;
+	  GNOME_Magnifier_RectBounds roi;
+	  roi.x1 = 100;
+	  roi.x2 = 100 + (screen_width / global_options.zoom_factor);
+	  roi.y1 = 0;
+	  roi.y2 = screen_height / global_options.zoom_factor;
+	  
+	  zoom_regions = GNOME_Magnifier_Magnifier_getZoomRegions (
+		  BONOBO_OBJREF (magnifier), &ev);
+	  
+	  if (zoom_regions && (zoom_regions->_length > 0))
+	  {
+		  GNOME_Magnifier_ZoomRegion_setROI (
+			  zoom_regions->_buffer[0], &roi, &ev);
+	  }
+  }
   else if (global_options.timing_iterations)
-    {
-      refresh_handle = gtk_timeout_add (global_options.refresh_time,
-                                        magnifier_main_test_image,
-                                        magnifier);
-    }
+  {
+	  refresh_handle = g_timeout_add (global_options.refresh_time,
+					  magnifier_main_test_image,
+					  magnifier);
+  }
   else
-    {
-      refresh_handle = gtk_timeout_add (
-        global_options.refresh_time,
-        magnifier_main_refresh_all, magnifier);
-
-      pan_handle = gtk_timeout_add (
-        global_options.mouse_poll_time,
-      magnifier_main_pan_image, magnifier);
-    }
-
+  {
+	  refresh_handle = g_timeout_add (
+		  global_options.refresh_time,
+		  magnifier_main_refresh_all, magnifier);
+	  
+	  pan_handle = g_timeout_add (
+		  global_options.mouse_poll_time,
+		  magnifier_main_pan_image, magnifier);
+  }
+  
   bonobo_main ();
   
   if (refresh_handle)
-      gtk_timeout_remove (refresh_handle);
-
+	  g_source_remove (refresh_handle);
+  
   if (pan_handle)
-	  gtk_timeout_remove (pan_handle);
+	  g_source_remove (pan_handle);
   
   return 0;
 }
