@@ -22,6 +22,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <popt.h>
 #include <gdk/gdkwindow.h>
 #include <libbonobo.h>
@@ -34,6 +35,8 @@
 #include "zoom-region-private.h"
 #include "GNOME_Magnifier.h"
 
+#define MAG_WINDOW_OVERRIDE_REDIRECT
+
 static GObjectClass *parent_class = NULL;
 
 enum {
@@ -77,6 +80,49 @@ magnifier_x_error_handler (Display	 *display,
 	return 0;
 }
 
+GdkFilterReturn
+magnifier_expose_filter (GdkXEvent *xevent, GdkEvent *event, gpointer data)
+{
+	Magnifier *magnifier = data;
+	GList *list;
+
+	if (event->any.type == GDK_EXPOSE) 
+	{
+		GNOME_Magnifier_RectBounds rect_bounds;
+		GdkRectangle rect;
+		gdk_region_get_clipbox (event->expose.region, &rect);
+		rect_bounds.x1 = rect.x;
+		rect_bounds.y1 = rect.y;
+		rect_bounds.x2 = rect.x + rect.width;
+		rect_bounds.y2 = rect.y + rect.height;
+		
+		g_assert (magnifier);
+		list = magnifier->zoom_regions;
+		
+		while (list) 
+		{
+			/* propagate the expose events to the zoom regions */
+			ZoomRegion *zoom_region;
+			CORBA_Environment ev;
+			zoom_region = list->data;
+			CORBA_exception_init (&ev);
+			GNOME_Magnifier_ZoomRegion_markDirty (BONOBO_OBJREF (zoom_region),
+							      &rect_bounds,
+							      &ev);
+			list = g_list_next (list);
+		}
+		return GDK_FILTER_TRANSLATE;
+	}
+	return GDK_FILTER_CONTINUE;
+}
+
+static void
+magnifier_set_expose_listener (Magnifier *magnifier, GdkWindow *root)
+{
+	/* todo: remove listener from 'old' root if one was already present */
+	gdk_window_add_filter (root, magnifier_expose_filter, NULL);
+}
+
 static void
 magnifier_realize (GtkWidget *widget)
 {
@@ -87,7 +133,8 @@ magnifier_realize (GtkWidget *widget)
 	GdkDisplay *target_display = gdk_drawable_get_display (widget->window);
 	
 	static gboolean initialized = FALSE;
-	
+
+#ifdef MAG_WINDOW_OVERRIDE_REDIRECT	
 	if (!initialized) {
 		wm_window_protocols[0] = gdk_x11_get_xatom_by_name_for_display (target_display,
 										"WM_DELETE_WINDOW");
@@ -117,13 +164,14 @@ magnifier_realize (GtkWidget *widget)
 				 XA_ATOM, 32, PropModeReplace,
 				 (guchar *)wm_type_atoms,
 				 1);
-
+#else
+#endif
 	/* TODO: make sure this works/is reset if the DISPLAY 
 	 * (as well as the SCREEN) changes.
 	 */
 
 	XSetErrorHandler (magnifier_x_error_handler);
-	fprintf (stderr, "Realize complete.\n");
+
 }
 
 GdkWindow*
@@ -258,11 +306,12 @@ magnifier_transform_cursor (Magnifier *magnifier)
 			magnifier->priv->w->window,
 			size, size,
 			1);
-		gdk_pixbuf_render_to_drawable (scaled_cursor_pixbuf,
-					       magnifier->priv->cursor,
-					       cgc,
-					       0, 0, 0, 0, size, size,
-					       GDK_RGB_DITHER_NONE, 0, 0 );
+
+		gdk_draw_pixbuf (magnifier->priv->cursor,
+				 cgc,
+				 scaled_cursor_pixbuf,
+				 0, 0, 0, 0, size, size,
+				 GDK_RGB_DITHER_NONE, 0, 0 );
 		scaled_mask_pixbuf = gdk_pixbuf_add_alpha (
 			scaled_mask_pixbuf, True, 0, 0, 0);
 		gdk_pixbuf_render_threshold_alpha (scaled_mask_pixbuf,
@@ -565,6 +614,9 @@ impl_magnifier_set_source_display (PortableServer_Servant servant,
 				ev,
 				magnifier);
 
+	/* attach listener to root window of source display */
+	magnifier_set_expose_listener (magnifier, magnifier_get_root (magnifier));
+
 	bonobo_arg_release (arg);
 }
 
@@ -649,6 +701,13 @@ impl_magnifier_add_zoom_region (PortableServer_Servant servant,
 {
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
 
+	if (magnifier->zoom_regions == NULL) 
+	{
+		/* first zoom region */
+		/* add an expose handler to root window; for XSun/XVirtualScreen 'dirty region' support */
+		magnifier_set_expose_listener (magnifier, magnifier_get_root (magnifier));
+	}
+
 	/* FIXME: this needs proper lifecycle management */
 	magnifier->zoom_regions = g_list_append (magnifier->zoom_regions, region);
 
@@ -878,7 +937,7 @@ magnifier_init_window (Magnifier *magnifier)
 		g_object_connect (gtk_widget_new (gtk_window_get_type (),
 						  "user_data", NULL,
 						  "can_focus", FALSE,
-						  "type", GTK_WINDOW_TOPLEVEL,
+						  "type", GTK_WINDOW_POPUP,
 						  "title", "magnifier",
 						  "allow_grow", TRUE,
 						  "allow_shrink", TRUE,
@@ -887,7 +946,6 @@ magnifier_init_window (Magnifier *magnifier)
 				  "signal::realize", magnifier_realize, NULL,
 				  "signal::destroy", magnifier_exit, NULL,
 				  NULL);
-	gtk_window_set_decorated (GTK_WINDOW (magnifier->priv->w), FALSE);
 	magnifier->priv->canvas = gtk_fixed_new ();
 	gtk_container_add (GTK_CONTAINER (magnifier->priv->w),
 			   magnifier->priv->canvas);
@@ -897,8 +955,6 @@ magnifier_init_window (Magnifier *magnifier)
 static void
 magnifier_init (Magnifier *magnifier)
 {
-	int ret;
-	
 	magnifier->priv = g_new0 (MagnifierPrivate, 1);
 	magnifier_properties_init (magnifier);
 	magnifier->zoom_regions = NULL;
@@ -916,9 +972,6 @@ magnifier_init (Magnifier *magnifier)
 	magnifier->priv->w = NULL;
 	magnifier_init_window (magnifier);
 	
-	ret = bonobo_activation_active_server_register (
-		MAGNIFIER_OAFIID, BONOBO_OBJREF (magnifier));
-
 	magnifier->cursor_set = "default";
 }
 
@@ -934,10 +987,21 @@ magnifier_get_cursor (Magnifier *magnifier)
 Magnifier *
 magnifier_new (void)
 {
-	return g_object_new (magnifier_get_type(), NULL);
+	Magnifier *mag;
+	int ret;
+
+	mag = g_object_new (magnifier_get_type(), NULL);
+	ret = bonobo_activation_active_server_register (
+		MAGNIFIER_OAFIID, BONOBO_OBJREF (mag));
+	if (ret != Bonobo_ACTIVATION_REG_SUCCESS)
+	    g_error ("Error registering magnifier server.\n");
+	else
+	    g_message ("The magnifier server was successfully registred.\n");
+	
+	return mag;
 }
 
 BONOBO_TYPE_FUNC_FULL (Magnifier, 
 		       GNOME_Magnifier_Magnifier,
 		       BONOBO_TYPE_OBJECT,
-		       magnifier);
+		       magnifier)
